//
// Copyright 2002 by Craig Stuart Sapp, All Rights Reserved.
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Mon Feb 11 19:48:50 PST 2002
// Last Modified: Sun Mar 24 12:10:00 PST 2002 (small changes for visual c++)
// Filename:      ...sig/src/sigInfo/ScoreRecord.cpp
// Web Address:   http://sig.sapp.org/include/sigInfo/ScoreRecord.cpp
// Syntax:        C++ 
//
// Description:   A SCORE item parameter class
//

#include "ScoreRecord.h"
#include "Convert.h"

#include <stdio.h>
#include <string.h>

#ifndef OLDCPP
   #include <iostream>
   #include <iomanip>
   #include <sstream>
   #define SSTREAM stringstream
   #define CSTRING str().c_str()
   using namespace std;
#else
   #include <iostream.h>
   #include <iomanip.h>
   #ifdef VISUAL
      #include <strstrea.h>
   #else
      #include <strstream.h>
   #endif
   #define SSTREAM strstream
   #define CSTRING str()
#endif
   

//////////////////////////////
//
// ScoreRecord::ScoreRecord --
//

ScoreRecord::ScoreRecord(void) : ParameterItem() { 
   // do nothing
   textdata = new char[1];
   textdata[0] = '\0';

   analysisData.setSize(3);
   analysisData.allowGrowth(0);
   analysisData.zero();
}



//////////////////////////////
//
// ScoreRecord::~ScoreRecord --
//

ScoreRecord::~ScoreRecord(void) { 
   clear();
   if (textdata != NULL) {
      delete [] textdata;
      textdata = NULL;
   }
}


//////////////////////////////
//
// ScoreRecord::clear --
//

void ScoreRecord::clear(void) {
   if (textdata != NULL) {
      delete [] textdata;
   }
   textdata = new char[1];
   textdata[0] = '\0';

   analysisData.setSize(3); 
   analysisData.allowGrowth(0);
   analysisData.zero();
}



//////////////////////////////
//
// ScoreRecord::printAscii -- print the parameter data in ASCII text
//     format.
//     default value: verboseQ = 0
//

void ScoreRecord::printAscii(ostream& out, int verboseQ) { 
   if (fixedParameters.getSize() < 1) {
      return;
   }

   union { float num; unsigned char ch[4]; } u;
   static char buffer[128] = {0};

   int i;
   for (i=0; i<fixedParameters.getSize(); i++) {
      sprintf(buffer, "%f", fixedParameters[i]);
      u.num = (float)atof(buffer);
      if (u.num != fixedParameters[i]) {
         SSTREAM stream;
         stream.precision(8);
         stream << fixedParameters[i];
         stream << ends;
         strcpy(buffer, stream.CSTRING);
      } 
      u.num = (float)atof(buffer);
      if (verboseQ) {
         if (u.num != fixedParameters[i]) {
            cout << "XXX:";
         }
      }
      adjustBuffer(buffer);
      if (strchr(buffer, 'e') != NULL) {
         strcpy(buffer, "0.0");
      }

      switch (i) {
         case 0: 
            cout << setiosflags(ios::left);
            if ((int)getValue(0) == 16) {
               out << setw(3) << "t";
               out << ' ';
            } else {
               out << setw(3) << buffer;
               if (i < fixedParameters.getSize()-1) {
                  out << ' ';
               }
            }
            break;
         case 1: 
            if (fixedParameters[i] >= 100) {
               out << setw(3) << buffer;
            } else {
               out << setw(2) << buffer;
            }
            if (i< fixedParameters.getSize()-1) {
               out << ' ';
            }
            break;
         case 2: 
            cout << setiosflags(ios::left);
            if (fixedParameters[i] < 10.0) {
               out << ' ';
            }
            if (fixedParameters[i] < 100.0) {
               out << ' ';
            }
            out << setw(10) << buffer;
            if (fixedParameters[i] >= 10.0) {
               out << ' ';
            }
            if (fixedParameters[i] >= 100.0) {
               out << ' ';
            }
            break;
         default: 
            out << buffer;
            if (i<fixedParameters.getSize()-1) {
               out << ' ';
            }
      }
   }

   if (isTextItem()) {
      out << "\n" << getTextData();
   }

   if (isNoteItem() && verboseQ && getPitch() != 0) {
      char buffer[32];
      out << "\n; pitch=" << getPitch() << " ("
          << Convert::base40ToKern(buffer, getPitch()) << ")";
   }


   // leave printing of final newline to the calling function
}



//////////////////////////////
//
// ScoreRecord::writeBinary -- print the parameter data in Binary
//    format, as found in a SCORE .mus file where the number of numbers
//    to be written is given first.
//

int ScoreRecord::writeBinary(ostream& out) { 
   if (fixedParameters.getSize() < 1) {
      return 0;
   }

   int i;
   writeLittleEndian(out, (float)fixedParameters.getSize());
   for (i=0; i<fixedParameters.getSize(); i++) {
      writeLittleEndian(out, fixedParameters[i]);
   }

   return fixedParameters.getSize() + 1;
}



//////////////////////////////
//
// ScoreRecord::readBinary -- input parameter values from the given
//     input stream.
//

void ScoreRecord::readBinary(istream& instream, int pcount) {
   int i;
   fixedParameters.setSize(pcount);
   if (pcount < 1) {
      cout << "Error: invalid parameter count: " << pcount << endl;
      exit(1);
   }
   setValue(0, readLittleEndian(instream));

   if ((int)getValue(0) == 16) {
      fixedParameters.setSize(13);
      for (i=1; i<13; i++) {
         setValue(i, readLittleEndian(instream));
      }
      int count = (int)getPValue(12);
      // char buffer[count+1+16];  // can't be done in MS Visual C++
      static char buffer[1024] = {0}; 
      instream.read(buffer, count);
      buffer[count] = '\0';
      setTextData(buffer);
      int extra = count % 4;
      if (extra > 0) {
         instream.read(buffer, 4-extra);
         for (i=0; i<4-extra; i++) {
            if (buffer[i] != 0x20) {
              cout << "Error: non space character found" << endl;
              exit(1);
            }
         }
      }
   } else {
      // non-text data parameters
      for (i=1; i<pcount; i++) {
         setValue(i, readLittleEndian(instream));
      }
   }
}



//////////////////////////////
//
// ScoreRecord::shrink --
//

void ScoreRecord::shrink(void) {
   int i;
   for (i=fixedParameters.getSize(); i>0; i--) {
      if (fixedParameters[i] != 0.0) {
         break;
      } else {
         fixedParameters.setSize(i);
      }
   }
}


//////////////////////////////////////////////////////////////////////////
//
// private functions
//


///////////////////////////////
//
// ScoreRecord::writeLittleEndian --
//

void ScoreRecord::writeLittleEndian(ostream& out, float number) {
   union { float f; unsigned int i; } num;
   num.f = number;
   char byteinfo[4];
   byteinfo[0] = (char)( num.i        & 0xff);
   byteinfo[1] = (char)((num.i >> 8)  & 0xff);
   byteinfo[2] = (char)((num.i >> 16) & 0xff);
   byteinfo[3] = (char)((num.i >> 24) & 0xff);
   out.write(byteinfo, 4);
}



///////////////////////////////
//
// ScoreRecord::readLittleEndian --
//

float ScoreRecord::readLittleEndian(istream& instream) {
   unsigned char byteinfo[4];
   instream.read((char*)byteinfo, 4);
   union { float f; unsigned int i; } num;
   num.i = 0;
   num.i = byteinfo[3];
   num.i = (num.i << 8) | byteinfo[2];
   num.i = (num.i << 8) | byteinfo[1];
   num.i = (num.i << 8) | byteinfo[0];

   return num.f;
}



//////////////////////////////
//
// ScoreRecord::adjustBuffer -- remove all right hand zeros
//

void ScoreRecord::adjustBuffer(char *buffer) {
   int len = strlen(buffer);
   int i;
   for (i=len-1; i>0; i--) {
      if (buffer[i] == '0') {
         buffer[i] = '\0';
      } else {
         break;
      }
   }
}



//////////////////////////////
//
// ScoreRecord::getTextData -- return the Code items text data or an
//     empty string.
//

const char* ScoreRecord::getTextData(void) {
   if (textdata == NULL) {
      return "";
   } else {
      return textdata;
   }
}



//////////////////////////////
//
// ScoreRecord::setTextData -- set the Code items text data field.
//

void ScoreRecord::setTextData(const char* buffer) {
   if (textdata != NULL) {
      delete [] textdata;
   } 
   int length = strlen(buffer);
   textdata = new char[length+1];
   strcpy(textdata, buffer);
}



//////////////////////////////
//
// ScoreRecord::getPitch --
//

int ScoreRecord::getPitch(void) { 
   return (int)analysisData[0];
}



//////////////////////////////
//
// ScoreRecord::setPitch --
//

void ScoreRecord::setPitch(int aPitch) { 
   analysisData[0] = (float)aPitch;
}



//////////////////////////////
//
// ScoreRecord::getStartOffset -- get the starting time from the start
//     of the staff.
//

float ScoreRecord::getStartOffset(void) { 
   return analysisData[1];
}



//////////////////////////////
//
// ScoreRecord::setStartOffset -- set the starting time from the start
//     of the staff
//

void ScoreRecord::setStartOffset(float aDuration) { 
   analysisData[1] = aDuration;
}



//////////////////////////////
//
// ScoreRecord::getVoice --
//

int ScoreRecord::getVoice(void) { 
   return (int)analysisData[2];

}



//////////////////////////////
//
// ScoreRecord::setVoice --
//

void ScoreRecord::setVoice(int aVoice) { 
   analysisData[2] = (float)aVoice;
}



//////////////////////////////
//
// ScoreRecord::is... -- boolean for code Item values.
//

int ScoreRecord::isNoteItem(void)       { return (int)getPValue(1)==1 ? 1 : 0;}
int ScoreRecord::isRestItem(void)       { return (int)getPValue(1)==2 ? 1 : 0;}
int ScoreRecord::hasDurationQ(void)     { return isNoteItem() || isRestItem();}
int ScoreRecord::isClefItem(void)       { return (int)getPValue(1)==3 ? 1 : 0;}
int ScoreRecord::isLineItem(void)       { return (int)getPValue(1)==4 ? 1 : 0;}
int ScoreRecord::isSlurItem(void)       { return (int)getPValue(1)==5 ? 1 : 0;}
int ScoreRecord::isBeamItem(void)       { return (int)getPValue(1)==6 ? 1 : 0;}
int ScoreRecord::isTrillItem(void)      { return (int)getPValue(1)==7 ? 1 : 0;}
int ScoreRecord::isStaffItem(void)      { return (int)getPValue(1)==8 ? 1 : 0;}
int ScoreRecord::isSymbolItem(void)     { return (int)getPValue(1)==9 ? 1 : 0;}
int ScoreRecord::isNumberItem(void)     { return (int)getPValue(1)==10? 1 : 0;}
int ScoreRecord::isUserItem(void)       { return (int)getPValue(1)==11? 1 : 0;}
int ScoreRecord::isShapeItem(void)      { return (int)getPValue(1)==12? 1 : 0;}
int ScoreRecord::isBarlineItem(void)    { return (int)getPValue(1)==14? 1 : 0;}
int ScoreRecord::isPostscriptItem(void) { return (int)getPValue(1)==15? 1 : 0;}
int ScoreRecord::isTextItem(void)       { return (int)getPValue(1)==16? 1 : 0;}
int ScoreRecord::isKeysigItem(void)     { return (int)getPValue(1)==17? 1 : 0;}
int ScoreRecord::isTimesigItem(void)    { return (int)getPValue(1)==18? 1 : 0;}



